#include "StdAfx.h"
#include "UtilFunction.h"
#include "ConvertUTF.h"
#include <ctype.h>
#if defined(WIN32)
#include "base64.hpp"
//#include "cxlib/ximage.h"
#include <ctype.h>
#include <NcMath.h>
#include <strstream>
#endif // WIN32

#include <algorithm>
#include "GameTalkAPIGlobal.h"

#if defined(WIN32)
#define STR_PATH_SEPERATOR	L'\\'
#else
#define STR_PATH_SEPERATOR	L'/'
#endif

#if defined(WIN32)
int RegistryUtil::ReadRegistry(LPCTSTR szCompany, LPCTSTR szAppName, LPCTSTR szSection, LPCTSTR szDataName, void * pDataValue, int nMaxSize, DWORD * pdwType, BOOL bLocalMachine)
{
	TCHAR KeyStr[256];
	HKEY hRegKey = NULL;

	_stprintf(KeyStr, _T("Software\\%s\\%s\\%s"), szCompany, szAppName, szSection);
	if(RegOpenKeyEx(bLocalMachine ? HKEY_LOCAL_MACHINE : HKEY_CURRENT_USER, KeyStr, 0, KEY_QUERY_VALUE, &hRegKey) != ERROR_SUCCESS)
		return 0;

	RegQueryValueEx(hRegKey, szDataName, 0, pdwType, (LPBYTE)pDataValue, (LPDWORD)&nMaxSize);
	RegCloseKey(hRegKey);
		
	return nMaxSize;
}
#endif // WIN32

bool UserUtil::GetIDAndDomain(WSTRING strFullID, WSTRING& strID, WSTRING& strDomain)
{
	for(size_t i = 0; i < GT_GLOBAL->GetEnvironment().vecAuthDomains.size(); i++)
	{
		WSTRING::size_type pos = strFullID.rfind(GT_GLOBAL->GetEnvironment().vecAuthDomains[i]);

		if( WSTRING::npos == pos )
			continue;

		strID = strFullID.substr(0, pos-1);
		strDomain = GT_GLOBAL->GetEnvironment().vecAuthDomains[i];
		return true;
	}

	strID = strFullID;
	strDomain = _T("");
	return false;
}



WSTRING FileUtil::ExtractFilePath(LPCWSTR strPath)
{
	WSTRING path(strPath);
	WSTRING::size_type spos = path.rfind(STR_PATH_SEPERATOR);

	WSTRING npath;
	if(spos != WSTRING::npos)
		npath = path.substr(0, spos);
	else
		npath = strPath;
	
	return npath;
}


WSTRING FileUtil::ExtractFileName(LPCWSTR strName)
{
	WSTRING name(strName);
	WSTRING::size_type spos = name.rfind(STR_PATH_SEPERATOR);

	WSTRING fname;
	if(spos != WSTRING::npos)
		fname = name.substr(spos+1, name.length());
	else
		fname = strName;
	
	return fname;
}

static bool __CreateDirectory(LPCWSTR szPath)
{
#if defined(WIN32)
	return CreateDirectory(szPath, 0) > 0 ? true : false;
#else
	return false;
#endif
}

bool FileUtil::ForceDirectory(LPCWSTR szPath)
{
#if defined (WIN32)
	WSTRING path = szPath;

	if(path.length() <= 0)
		return false;

	std::transform(path.begin(), path.end(), path.begin(), towupper);  //Make uppercase
	
	if (!((path[0] >= 'C' && path[0] <='Z') && path[1] == ':' && path[2] == '\\'))
		return false;

	if(path.at(path.length() - 1) == _T('\\'))
		path.erase(path.length() - 1);

	int code = GetFileAttributes(path.c_str());
	
	if (code == -1)
	{
		if (path.find(STR_PATH_DELIMITER) == WSTRING::npos)
			return false;
	}

	if((code != -1) && ((FILE_ATTRIBUTE_DIRECTORY & code) != 0))
		return true;

	return (ForceDirectory(ExtractFilePath(path.c_str()).c_str()) && __CreateDirectory(path.c_str()));
#else
	
	return false;
#endif // WIN32
}


WSTRING UserUtil::GetIDExceptDomain(WSTRING strFullID)
{
	WSTRING strID;
	WSTRING strDomain;

	if(GetIDAndDomain(strFullID, strID, strDomain))
		return strID;
	return strFullID;
}

#if defined(WIN32)
int ConvertUtil::Text2Binary(const char * pText, int nSize, void * pOutBuffer)
{
	std::strstream src((char *)pText, nSize);

	std::istreambuf_iterator<char> _From(src.rdbuf());
	std::istreambuf_iterator<char> _To(0);


	std::strstreambuf buf;
	std::basic_iostream<char, std::char_traits<char> > ostream(&buf);
	std::ostreambuf_iterator<char> _Out(ostream);

	base64<char> encoder;
	int _State = 0;

	encoder.get(_From, _To, _Out, _State);

	ostream.seekg(0, std::ios_base::end);
	int nCount = ostream.tellg();
	ostream.seekg(0, std::ios_base::beg);
	ostream.read((char *)pOutBuffer, nCount);
	return nCount;
}
#endif

bool ConvertUtil::ConvertPhoneStyle(WSTRING &strResult, LPCTSTR szInput)
{
	bool bRet = false;
	const TCHAR * pch = szInput;
	while( *pch != '\0' )
	{
		if( isdigit( *pch ) || *pch == '#' || *pch == '*' || *pch == '@' || *pch == '+' )
		{
			strResult += *pch;
			bRet = true;
		}
		pch++;
	}

	return bRet;
}


AV_USAGE_MODE ConvertUtil::ConvertToInternalAVUsageMode(GTAVUsageMode avUsageMode)
{
	return (AV_USAGE_MODE)avUsageMode;
}

GTAVUsageMode ConvertUtil::ConvertToAPIAVUsageMode(AV_USAGE_MODE avUsageMode)
{
	return (GTAVUsageMode)avUsageMode;
}

COMMON_OS_TYPE ConvertUtil::ConvertToInternalOSType(GTOSType osType)
{
	return (COMMON_OS_TYPE)osType;
}

GTOSType ConvertUtil::ConvertToAPIOSType(COMMON_OS_TYPE osType)
{
	return (GTOSType)osType;
}


void StringUtil::StringFormatA(std::string &strResult, char *pszFormat, ...)
{
	char *pBuffer = NULL;
	va_list argList;
	va_start( argList, pszFormat );
	int nLength = vsnprintf( NULL, 0, pszFormat, argList );
	if(nLength <= 0)
		return;
	pBuffer = new char[nLength+1];
	if (NULL == pBuffer)
	{
		return;
	}

	vsnprintf( pBuffer, nLength, pszFormat, argList );
	pBuffer[nLength] = '\0';
	va_end( argList );
	
	strResult = pBuffer;
	delete []pBuffer;
	return;
}	


void StringUtil::StringFormatW(WSTRING &strResult, const WCHAR *pszFormat, ...)
{
#if defined(ANDROID)	// KEH_ANDROID_UNICODE
	// SAME WITH StringFormatA..

	char *pBuffer = NULL;
	va_list argList;
	va_start( argList, pszFormat );
	int nLength = vsnprintf( NULL, 0, pszFormat, argList );
	if(nLength <= 0)
		return;
	pBuffer = new char[nLength+1];
	vsnprintf( pBuffer, nLength, pszFormat, argList );
	pBuffer[nLength] = '\0';
	va_end( argList );
	
	strResult = pBuffer;
	delete []pBuffer;
	return;
#else
	WCHAR *pBuffer = NULL;
	va_list argList;
	va_start( argList, pszFormat );
	int nLength = vswprintf( NULL, 0, pszFormat, argList );
	if(nLength <= 0)
		return;
	pBuffer = new WCHAR[nLength+1];
	if (NULL == pBuffer)
		return;

	vswprintf( pBuffer, nLength, pszFormat, argList );
	pBuffer[nLength] = '\0';
	va_end( argList );
	
	strResult = pBuffer;
	delete []pBuffer;
	return;
#endif
}

int StringUtil::wtoi(const WCHAR *str)
{
	USES_UTF8_CONVERSION
	
	const char * p = W2U8(str);
	return atoi(p);
}

void StringUtil::itoa(int value, std::string& buf, int base)
{	
	int i = 30;	
	buf = "";
		
	if(value <= 0)
	{
		buf = "0";
		return;
	}
	
	for(; value && i ; --i, value /= base) buf = "0123456789abcdef"[value % base] + buf;
}

void StringUtil::itow(int value, WSTRING& buf, int base)
{	
#if defined(ANDROID)	// KEH_ANDROID_UNICODE
	StringUtil::itoa(value, buf, base);
#else
	int i = 30;	
	buf = L"";
	if(value <= 0)
	{
		buf = L"0";
		return;
	}
	
	for(; value && i ; --i, value /= base)
	{
		buf = L"0123456789abcdef"[value % base] + buf;
	}
#endif
}

// trim from start
std::string &StringUtil::ltrim(std::string &s) 
{
#if defined(WIN32)
	s.erase(s.begin(), std::find_if(s.begin(), s.end(), std::not1(std::ptr_fun<int, int>(isspace))));
#else
	s.erase(s.begin(), std::find_if(s.begin(), s.end(), std::not1(std::ptr_fun<int, int>(isspace))));
#endif
	return s;
}

// trim from end
std::string &StringUtil::rtrim(std::string &s) 
{
#if defined(WIN32)
	s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<int, int>(isspace))).base(), s.end());
#else
	s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<int, int>(isspace))).base(), s.end());
#endif
	return s;
}

// trim from both ends
std::string &StringUtil::trim(std::string &s) 
{
	return ltrim(rtrim(s));
}


//bool ImageUtil::GetImageCRCFromString(const char *strCRCString, DWORD &dwImgCRC)
//{
//    if(NULL == strCRCString || strlen(strCRCString) <= 0)
//    {
//        dwImgCRC = 0;
//        return false;
//    }
//    
//    unsigned int temp;
//    
//    sscanf(strCRCString, "%x", &temp);
// 
//    dwImgCRC = temp;
//    
//    return true;
//}
//
//
//#if defined(WIN32)
//HBITMAP ImageUtil::LoadImage(LPCWSTR pszFileName, bool bDibSection, void ** ppDIBits)
//{
//	if(pszFileName == NULL || _tcslen(pszFileName) == 0)
//		return NULL;
//
//	CxImage image;
//	if(image.Load(pszFileName) == false)
//		return NULL;
//
//	HDC hDC = GetDC(NULL);
//	if(hDC == NULL)
//		return NULL;
//
//	HDC hMemDC;
//    HBITMAP hBitmap  = NULL;
//	HBITMAP hOldBitmap = NULL;
//	do
//	{
//		hMemDC = CreateCompatibleDC(hDC);
//		if(hMemDC == NULL)
//			break;
//
//		if(bDibSection)
//		{
//			BITMAPINFOHEADER bmih;
//			bmih.biSize = sizeof (BITMAPINFOHEADER);
//			bmih.biWidth = image.GetWidth();
//			bmih.biHeight = image.GetHeight();
//			bmih.biPlanes = 1;
//			bmih.biBitCount = 24;
//			bmih.biCompression = BI_RGB;
//			bmih.biSizeImage = DIBSIZE(bmih);//bmih.biWidth * bmih.biHeight * bmih.biBitCount / 8;
//			bmih.biXPelsPerMeter = 0;
//			bmih.biYPelsPerMeter = 0;
//			bmih.biClrUsed = 0;
//			bmih.biClrImportant = 0;
//			hBitmap = CreateDIBSection(hDC, (const BITMAPINFO*)&bmih, DIB_RGB_COLORS, ppDIBits, NULL, 0) ;
//		} else
//			hBitmap = CreateCompatibleBitmap(hDC, image.GetWidth(), image.GetHeight());
//		if(hBitmap == NULL)
//			break;
//
//		hOldBitmap = (HBITMAP)SelectObject(hMemDC, hBitmap);
//		image.Draw(hMemDC);
//	} while(FALSE);
//
//	if(hOldBitmap)
//		SelectObject(hMemDC, hOldBitmap);
//	if(hMemDC)
//		DeleteDC(hMemDC);
//	if(hDC)
//		ReleaseDC(NULL, hDC);
//
//	return hBitmap;
//}
//
//
//
//bool ImageUtil::ResizeImage(const BYTE* pSrcBuffer, // <IN>
//							int nSrcWidth, 
//							int nSrcHeight, 
//							int nSrcBytesPerPixel, 
//							BYTE* pDestBuffer,		// <OUT>
//							int nDestWidth, 
//							int nDestHeight, 
//							int nDestBytesPerPixel, 
//							bool bInterpolation)
//{
//	if (nSrcBytesPerPixel!=3 || nDestBytesPerPixel!=3)
//	{
//		ImageUtil::SmoothResize(pSrcBuffer, nSrcWidth, nSrcHeight, nSrcBytesPerPixel, pDestBuffer, nDestWidth, nDestHeight, nDestBytesPerPixel);
//		return true;
//	}
//	IppiSize szSrc, szDest;
//	IppiRect rtSrc;
//	int nSrcBytesPerLine;
//	int nDestBytesPerLine;
//	nSrcBytesPerLine = WIDTHBYTES(nSrcWidth*nSrcBytesPerPixel*8);
//	nDestBytesPerLine = WIDTHBYTES(nDestWidth*nDestBytesPerPixel*8);
//	szSrc.height	= nSrcHeight;
//	szSrc.width		= nSrcWidth;
//	rtSrc.x			= 0;
//	rtSrc.y			= 0;
//	rtSrc.width		= szSrc.width;
//	rtSrc.height	= szSrc.height;
//	szDest.width	= nDestWidth;
//	szDest.height	= nDestHeight;
//
//	double dXfactor, dYfactor;
//	dXfactor = 1.0*szDest.width/szSrc.width;
//	dYfactor = 1.0*szDest.height/szSrc.height;
//
//	int nInterpolation = IPPI_INTER_NN;
//	if (bInterpolation)
//	{
//		if (nSrcWidth>nDestWidth && nSrcHeight>nDestHeight)//Super sampling interpolation ҽÿ 밡
//		{
//			nInterpolation = IPPI_INTER_SUPER;
//		}
//		else
//		{
//			nInterpolation = IPPI_INTER_LINEAR;
//		}
//		//nInterpolation = IPPI_INTER_LANCZOS;
//		//nInterpolation = IPPI_INTER_CUBIC;
//	}
//	else
//	{
//		nInterpolation = IPPI_INTER_NN;
//	}
//
//	ippiResize_8u_C3R(pSrcBuffer, szSrc, nSrcBytesPerLine, rtSrc, 
//		pDestBuffer, nDestBytesPerLine, szDest, dXfactor, dYfactor, nInterpolation);
//	return false;
//}
//
//
//void ImageUtil::SmoothResize(const BYTE * image_in,  // <IN>
//							 int width_in, 
//							 int height_in, 
//							 int byteperpixel_in, 
//							 BYTE * image_out,		 // <OUT>
//							 int width_out, 
//							 int height_out, 
//							 int byteperpixel_out)
//{
//	int x, y, xp, yp, xp2, yp2;
//	const BYTE* pRead;
//	const BYTE* pRead2;
//	int t, t3, t13, z, z2, iz2;
//	PBYTE pc;
//	int w1, w2, w3, w4;
//	BYTE col1r, col1g, col1b, col2r, col2g, col2b;
//
//	xp2 = ((width_in - 1) << 15) / width_out;
//	yp2 = ((height_in - 1) << 15) / height_out;
//	yp = 0;
//	
//	for(y = 0; y < height_out; y++)
//	{
//		xp = 0;
//		pRead = image_in + (yp >> 15) * byteperpixel_in * width_in;
//		if (yp >> 16 < height_in - 1)
//			pRead2 = image_in + ((yp >> 15) + 1) * byteperpixel_in * width_in;
//		else
//			pRead2 = image_in + (yp >> 15) * byteperpixel_in * width_in;
//		pc = image_out + y * byteperpixel_out * width_out;
//		z2 = yp & 0x7fff;
//		iz2 = 0x8000 - z2;
//
//		for(x = 0; x < width_out; x++)
//		{
//			t = xp >> 15;
//			t3 = t * byteperpixel_in;
//			t13 = t3 + byteperpixel_in;
//			col1r = pRead[t3];
//			col1g = pRead[t3 + 1];
//			col1b = pRead[t3 + 2];
//			col2r = pRead2[t3];
//			col2g = pRead2[t3 + 1];
//			col2b = pRead2[t3 + 2];
//			z = xp & 0x7fff;
//			w2 = (z * iz2) >> 15;
//			w1 = iz2 - w2;
//			w4 = (z * z2) >> 15;
//			w3 = z2 - w4;
//			pc[x * byteperpixel_out + 2] = (col1b * w1 + pRead[t13 + 2] * w2 + col2b * w3 + (pRead2[t13 + 2] * w4)) >> 15;
//			pc[x * byteperpixel_out + 1] = (col1g * w1 + pRead[t13 + 1] * w2 +	col2g * w3 + (pRead2[t13 + 1] * w4)) >> 15;
//			pc[x * byteperpixel_out + 0] = (col1r * w1 + pRead[t13 + 0] * w2 + col2r * w3 + (pRead2[t13 + 0] * w4)) >> 15;
//			xp += xp2;
//		}
//		yp += yp2;
//	}
//}
//#endif // WIN32


GTPenaltyReasonType ParseUtil::GetPenaltyReasonFromNumber(int nReason)
{
    GTPenaltyReasonType ePenaltyReasonType;
    switch(nReason)
    {
		case 1:							ePenaltyReasonType = penaltySlang;				break;	
		case 2:							ePenaltyReasonType = penaltyTrySelling;			break;	
		case 3:							ePenaltyReasonType = penaltyCommercial;			break;	
		case 4:							ePenaltyReasonType = penaltyCommercialForAdult;	break;	
		case 5:							ePenaltyReasonType = penaltyAbnormalChat;		break;	
		case 6:							ePenaltyReasonType = penaltyFalseAdmin;			break;	
		case 7:							ePenaltyReasonType = penaltyPersonalInfoExposal;break;	
		case 8:							ePenaltyReasonType = penaltyDefamation;			break;	
		case 9:							ePenaltyReasonType = penaltyBombChat;			break;	
		case 100:						ePenaltyReasonType = penaltyDdosAccessDenied;	break;
		default:						ePenaltyReasonType = penaltyEtc;				break;	
    }
	return ePenaltyReasonType;
}

GTPenaltyStatusType ParseUtil::GetPenaltyStatusFromNumber(int nStatus)
{
	switch(nStatus)
	{
		case 0:		return penaltyStatus_None;
		case 6:
		case 9999:
			return penaltyStatus_Unlimited;
		case 1:		return penaltyStatus_1TimeWarning;
		case 2:		return penaltyStatus_2TimeWarning;
		case 3:		return penaltyStatus_3TimeWarning;
		case 101:	return penaltyStatus_1Day;	
		case 1001:	return penaltyStatus_1DayWaitRoom;
		case 107:	return penaltyStatus_7Day;	
		case 130:	return penaltyStatus_30Day;
		default:	return penaltyStatus_None;	
	}
	
	return penaltyStatus_None;
}
bool NetworkUtil::IPAddressSplit(LPCSTR pszServerAddr, std::string &strAddress, int &nPort)
{
	// Parse Server Address (xxx.xxx.xxx.xxx:xxx)
	if(strlen(pszServerAddr) > 50)
		return false;

	char szHostAddr[100];
	int nPortNum = -1;
	strcpy(szHostAddr, pszServerAddr);
	char * pszColon = strchr(szHostAddr, ':');
	if(pszColon)
	{
		*pszColon = 0;
		nPortNum = atoi(pszColon + 1);
	}
	if(nPortNum == 0)
		return false;

	strAddress = szHostAddr;
	nPort = nPortNum;

	return true;
}


const std::string Conversion::WideToUtf8(const std::wstring& wideString)
{
	int length = ::WideCharToMultiByte(CP_UTF8, 0, wideString.c_str(), wideString.size(), NULL, 0, NULL, NULL);
	char *szUtf8String = (char *) malloc(length);
	if (NULL == szUtf8String)
	{
		return "";
	}

	::WideCharToMultiByte(CP_UTF8, 0, wideString.c_str(), wideString.size(), szUtf8String, length, NULL, NULL);
	std::string utf8String(szUtf8String, length);
	free(szUtf8String);

	return utf8String;
}

const std::wstring Conversion::Utf8ToWide(const std::string& utf8String)
{
	int length = ::MultiByteToWideChar(CP_UTF8, 0, utf8String.c_str(), utf8String.size(), NULL, 0);
	if (0 == length)
	{
		return _T("");
	}
	std::vector<wchar_t> szWideString;
	szWideString.resize(length);
	::MultiByteToWideChar(CP_UTF8, 0, utf8String.c_str(), utf8String.size(), &*szWideString.begin(), length);
	std::wstring wideString;
	wideString.assign(szWideString.begin(), szWideString.end());

	return wideString;
}

//const std::wstring Conversion::AcpToWide(const std::string& utf8String)
//{
//	int length = ::MultiByteToWideChar(CP_ACP, 0, utf8String.c_str(), utf8String.size(), NULL, 0);
//	if (0 == length)
//		return _T("");
//
//	std::vector<wchar_t> szWideString;
//	szWideString.resize(length);
//	::MultiByteToWideChar(CP_ACP, 0, utf8String.c_str(), utf8String.size(), &*szWideString.begin(), length);
//	std::wstring wideString;
//	wideString.assign(szWideString.begin(), szWideString.end());
//
//	return wideString;
//}

//const std::string Conversion::WideToAcp(const std::wstring& wideString)
//{
//	int length = ::WideCharToMultiByte(CP_ACP, 0, wideString.c_str(), wideString.size(), NULL, 0, NULL, NULL);
//	if (0 == length)
//	{
//		return "";
//	}
//	char *szUtf8String = (char *) malloc(length);
//	if (NULL == szUtf8String)
//		return "";
//
//	::WideCharToMultiByte(CP_UTF8, 0, wideString.c_str(), wideString.size(), szUtf8String, length, NULL, NULL);
//	std::string utf8String(szUtf8String, length);
//	free(szUtf8String);
//
//	return utf8String;
//}


static char _x2c(char hex_up, char hex_low) 
{
	char digit;

	digit = 16 * (hex_up >= 'A' 
		? ((hex_up & 0xdf) - 'A') + 10 : (hex_up - '0'));
	digit += (hex_low >= 'A' 
		? ((hex_low & 0xdf) - 'A') + 10 : (hex_low - '0'));
	return (digit);
}

char *Conversion::URLdecode(char *str) 
{
	int i, j;

	if(!str) return NULL;

	for(i = j = 0; str[j]; i++, j++) 
	{
		switch(str[j]) 
		{
		case '+':
			str[i] = ' ';
			break;

		case '%':
			str[i] = _x2c(str[j + 1], str[j + 2]);
			j += 2;
			break;

		default:
			str[i] = str[j];
			break;
		}
	}
	str[i]='\0';

	return str;
}

char *Conversion::URLencode(char *str)
{
	char *encstr, buf[2+1];
	unsigned char c;
	int i, j;

	if(str == NULL) return NULL;
	if((encstr = (char *)malloc((strlen(str) * 3) + 1)) == NULL) 
		return NULL;

	for(i = j = 0; str[i]; i++) 
	{
		c = (unsigned char)str[i];
		if((c >= '0') && (c <= '9')) encstr[j++] = c;
		else if((c >= 'A') && (c <= 'Z')) encstr[j++] = c;
		else if((c >= 'a') && (c <= 'z')) encstr[j++] = c;
		else if((c == '@') || (c == '.') || (c == '/') || (c == '\\')
			|| (c == '-') || (c == '_') || (c == ':') ) 
			encstr[j++] = c;
		else 
		{
			sprintf(buf, "%02x", c);
			encstr[j++] = '%';
			encstr[j++] = buf[0];
			encstr[j++] = buf[1];
		}
	}
	encstr[j] = '\0';

	return encstr;
}
